import asyncio
import logging
from asyncio import get_event_loop
from typing import Dict, Optional

import attr
import click

from aapns import config, models
from aapns.api import create_client

SERVERS: Dict[bool, Dict[bool, config.Server]] = {
    True: {True: config.ProductionAltPort, False: config.Production},
    False: {True: config.DevelopmentAltPort, False: config.Development},
}

LOCAL_SERVERS: Dict[bool, config.Server] = {
    False: config.Server("localhost", 443),
    True: config.Server("localhost", 2197),
}


@attr.s
class Context:
    token = attr.ib()
    cert = attr.ib()
    server = attr.ib()
    apns_id = attr.ib()
    expiration = attr.ib()
    priority = attr.ib()
    topic = attr.ib()
    collapse_id = attr.ib()
    verbose = attr.ib()


async def do_send(context: Context, notification: models.Notification) -> Optional[str]:
    client = await create_client(
        context.cert,
        context.server,
        **{"cafile": "tests/functional/test-server-certificate.pem"}
        if context.server.host == "localhost"
        else {},
    )
    try:
        resp_id = await client.send_notification(
            context.token,
            notification,
            apns_id=context.apns_id,
            expiration=context.expiration,
            priority=context.priority,
            topic=context.topic,
            collapse_id=context.collapse_id,
        )
    finally:
        await client.close()
    return resp_id


def send(context: Context, notification: models.Notification):
    resp_id = asyncio.run(do_send(context, notification))
    click.echo(resp_id)


@click.group()
@click.argument("token")
@click.option("--client-cert-path", envvar="CLIENT_CERT_PATH")
@click.option("--prod", is_flag=True, default=False)
@click.option("--alt-port", is_flag=True, default=False)
@click.option("--expiration", default=None)
@click.option("--immediately", is_flag=True, default=False)
@click.option("--topic", default=None)
@click.option("--collapse-id", default=None)
@click.option("--apns-id", default=None)
@click.option("--verbose", is_flag=True, default=False)
@click.option("--local", is_flag=True, default=False)
@click.pass_context
def main(
    ctx,
    token,
    client_cert_path,
    prod,
    alt_port,
    expiration,
    immediately,
    topic,
    collapse_id,
    apns_id,
    verbose,
    local,
):
    logging.basicConfig(level=logging.DEBUG if verbose else logging.INFO)
    ctx.obj = Context(
        token=token,
        cert=client_cert_path,
        server=LOCAL_SERVERS[alt_port] if local else SERVERS[prod][alt_port],
        expiration=expiration,
        priority=config.Priority.immediately if immediately else config.Priority.normal,
        topic=topic,
        collapse_id=collapse_id,
        apns_id=apns_id,
        verbose=verbose,
    )


@main.command("simple")
@click.argument("body")
@click.option("--title", default=None)
@click.pass_context
def simple(ctx, title, body):
    try:
        notification = models.Notification(alert=models.Alert(title=title, body=body))
        send(ctx.obj, notification)
    except Exception:
        logging.exception("Simple notification")


@main.command("localized")
@click.argument("body")
@click.option("--body-args", multiple=True)
@click.option("--title", default=None)
@click.option("--title-args", multiple=True)
@click.option("--badge", type=click.INT)
@click.pass_context
def localized(ctx, title, body, title_args, body_args, badge):
    try:
        notification = models.Notification(
            alert=models.Alert(
                body=models.Localized(body, list(body_args)),
                title=models.Localized(title, list(title_args)) if title else title,
            ),
            badge=badge,
        )
        send(ctx.obj, notification)
    except Exception:
        logging.exception("Localised notification")



from typing import Optional

import attr

from . import config, errors, models
from .pool import Pool, Request, create_ssl_context


@attr.s(auto_attribs=True, frozen=True)
class APNS:
    server: config.Server
    pool: Pool

    async def send_notification(
        self,
        token: str,
        notification: models.Notification,
        *,
        apns_id: Optional[str] = None,
        expiration: Optional[int] = None,
        priority: config.Priority = config.Priority.normal,
        topic: Optional[str] = None,
        collapse_id: Optional[str] = None,
    ) -> Optional[str]:

        r = Request.new(
            path=f"/3/device/{token}",
            header={
                "apns-priority": str(priority.value),
                "apns-push-type": notification.push_type.value,
                **({"apns-id": apns_id} if apns_id else {}),
                **({"apns-expiration": str(expiration)} if expiration else {}),
                **({"apns-topic": topic} if topic else {}),
                **({"apns-collapse-id": collapse_id} if collapse_id else {}),
            },
            data=notification.get_dict(),
            timeout=10,
        )
        response = await self.pool.post(r)
        if response.code != 200:
            raise errors.get(response.reason, response.apns_id)
        return response.apns_id

    async def close(self):
        await self.pool.close()


async def create_client(
    client_cert_path: str, server: config.Server, *, cafile: Optional[str] = None,
) -> APNS:
    base_url = f"https://{server.host}:{server.port}"
    ssl_context = create_ssl_context()
    if cafile:
        ssl_context.load_verify_locations(cafile=cafile)
    ssl_context.load_cert_chain(certfile=client_cert_path, keyfile=client_cert_path)
    return APNS(server, await Pool.create(base_url, ssl=ssl_context))



import json
from enum import Enum, unique
from typing import *

import attr


@attr.s
class Localized:
    key: str = attr.ib(validator=attr.validators.instance_of(str))
    args: Optional[List[str]] = attr.ib(
        default=None,
        validator=attr.validators.optional(
            attr.validators.deep_iterable(
                attr.validators.instance_of(str), attr.validators.instance_of(list)
            )
        ),
    )


MaybeLocalized = Union[Dict[str, str], Dict[str, Union[List[str], str]]]


def maybe_localized(
    thing: Union[str, Localized], nonloc: str, lockey: str, locarg: str
) -> MaybeLocalized:
    if isinstance(thing, Localized):
        attr.validate(thing)
        localized: Dict[str, Union[str, List[str]]] = {lockey: thing.key}
        if thing.args:
            localized[locarg] = thing.args
        return localized
    else:
        return {nonloc: thing}


@attr.s
class Alert:
    body: Union[str, Localized] = attr.ib(
        validator=attr.validators.instance_of((str, Localized))
    )
    title: Optional[Union[str, Localized]] = attr.ib(
        default=None,
        validator=attr.validators.optional(
            attr.validators.instance_of((str, Localized))
        ),
    )
    action_loc_key: Optional[str] = attr.ib(
        default=None,
        validator=attr.validators.optional(attr.validators.instance_of(str)),
    )
    launch_image: Optional[str] = attr.ib(
        default=None,
        validator=attr.validators.optional(attr.validators.instance_of(str)),
    )

    def get_dict(self) -> Dict[str, Any]:
        attr.validate(self)
        alert: Dict[str, Any] = {}
        if self.title:
            alert.update(
                maybe_localized(self.title, "title", "title-loc-key", "title-loc-args")
            )
        alert.update(maybe_localized(self.body, "body", "loc-key", "loc-args"))
        if self.action_loc_key:
            alert["action-loc-key"] = self.action_loc_key
        if self.launch_image:
            alert["launch-image"] = self.launch_image
        return alert


@unique
class PushType(Enum):
    alert = "alert"
    background = "background"


@attr.s
class Notification:
    alert: Alert = attr.ib(validator=attr.validators.instance_of(Alert))
    push_type: PushType = attr.ib(
        default=PushType.alert, validator=attr.validators.instance_of(PushType)
    )
    badge: Optional[int] = attr.ib(
        default=None,
        validator=attr.validators.optional(attr.validators.instance_of(int)),
    )
    sound: Optional[str] = attr.ib(
        default=None,
        validator=attr.validators.optional(attr.validators.instance_of(str)),
    )
    content_available: bool = attr.ib(
        default=False, validator=attr.validators.instance_of(bool)
    )
    category: Optional[str] = attr.ib(
        default=None,
        validator=attr.validators.optional(attr.validators.instance_of(str)),
    )
    thread_id: Optional[str] = attr.ib(
        default=None,
        validator=attr.validators.optional(attr.validators.instance_of(str)),
    )
    extra: Optional[Dict[str, Any]] = attr.ib(
        default=None,
        validator=attr.validators.optional(
            attr.validators.deep_mapping(
                attr.validators.instance_of(str),
                attr.validators.instance_of(object),
                attr.validators.instance_of(dict),
            )
        ),
    )

    def get_dict(self) -> Dict[str, Any]:
        attr.validate(self)
        apns: Dict[str, Any] = {"alert": self.alert.get_dict()}
        if self.badge:
            apns["badge"] = self.badge
        if self.sound:
            apns["sound"] = self.sound
        if self.content_available:
            apns["content-available"] = 1
        if self.category:
            apns["category"] = self.category
        if self.thread_id:
            apns["thread-id"] = self.thread_id
        raw = {"aps": apns}
        if self.extra:
            raw.update(self.extra)
        return raw

    def encode(self) -> bytes:
        raw = self.get_dict()
        s = json.dumps(raw, ensure_ascii=False, separators=(",", ":"), sort_keys=True)
        return s.encode("utf-8")



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['aapns']

package_data = \
{'': ['*']}

install_requires = \
['attrs>=19.3.0,<20.0.0', 'h2>=3.2.0,<4.0.0']

extras_require = \
{'cli': ['click>=7.0,<8.0']}

entry_points = \
{'console_scripts': ['aapns = aapns.cli:main']}

setup_kwargs = {
    'name': 'aapns',
    'version': '20.4',
    'description': 'Asynchronous Apple Push Notification Service Client',
    'long_description': "# AAPNS\n\n[![CircleCI](https://circleci.com/gh/HENNGE/aapns.svg?style=svg)](https://circleci.com/gh/HENNGE/aapns)\n[![Documentation Status](https://readthedocs.org/projects/aapns/badge/?version=latest)](http://aapns.readthedocs.io/en/latest/?badge=latest)\n\nAsynchronous Apple Push Notification Service client.\n\n* Requires TLS 1.2 or better\n* Requires Python 3.8 or better\n\n## Quickstart\n\n```python\nfrom aapns.api import create_client\nfrom aapns.config import Priority, Production\nfrom aapns.models import Notification, Alert, Localized\n\nasync def send_hello_world():\n    client = await create_client('/path/to/push/cert.pem', Production)\n    apns_id = await client.send_notification(\n        'my-device-token',\n        Notification(\n            alert=Alert(\n                body=Localized(\n                    key='Hello World!',\n                    args=['foo', 'bar']\n                ),\n            ),\n            badge=42\n        ),\n        priority=Priority.immediately\n    )\n    print(f'Sent push notification with ID {apns_id}')\n    await client.close()\n```\n",
    'author': 'Jonas Obrist',
    'author_email': 'jonas.obrist@hennge.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/HENNGE/aapns',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
